@@ -1,4 +1,5 @@
 use std::io::Result;
+use std::net::SocketAddr;
 use socket2::{Socket, Domain, Type};
 
 /// Create a new non-blocking socket.
@@ -38,7 +39,7 @@ pub fn new_socket(domain: Domain, ty: Type) -> Result<Socket> {
     }
 }
 
-/// Create a new socket. 
+/// Create a new non-blocking socket.
 #[cfg(not(any(
     target_os = "android",
     target_os = "dragonfly",
@@ -55,6 +56,41 @@ pub fn new_socket(domain: Domain, ty: Type) -> Result<Socket> {
     Ok(socket)
 }
 
+/// Create a new non-blocking TCP socket.
+/// 
+/// On unix-like platforms, [`SOCK_NONBLOCK`](libc::SOCK_NONBLOCK) and
+/// [`SOCK_CLOEXEC`](libc::SOCK_CLOEXEC) are assigned
+/// when creating a new socket, which saves a [`fcntl`](libc::fcntl) syscall.
+/// 
+/// On other platforms, a socket is created without extra flags
+/// then set to `non_blocking`.
+#[inline]
+pub fn new_tcp_socket(addr: &SocketAddr) -> Result<Socket> {
+    let domain = match addr {
+        SocketAddr::V4(..) => Domain::IPV4,
+        SocketAddr::V6(..) => Domain::IPV6,
+    };
+    new_socket(domain, Type::STREAM)
+}
+
+/// Create a new non-blocking UDP socket.
+/// 
+/// On unix-like platforms, [`SOCK_NONBLOCK`](libc::SOCK_NONBLOCK) and
+/// [`SOCK_CLOEXEC`](libc::SOCK_CLOEXEC) are assigned
+/// when creating a new socket, which saves a [`fcntl`](libc::fcntl) syscall.
+/// 
+/// On other platforms, a socket is created without extra flags
+/// then set to `non_blocking`.
+#[inline]
+pub fn new_udp_socket(addr: &SocketAddr) -> Result<Socket> {
+    let domain = match addr {
+        SocketAddr::V4(..) => Domain::IPV4,
+        SocketAddr::V6(..) => Domain::IPV6,
+    };
+    new_socket(domain, Type::DGRAM)
+}
+
+
 /// Bind a socket to a specific network interface.
 /// 
 /// It seems `SO_BINDTODEVICE` is not supported on BSDs, we should use `IP_SENDIF` instead.
